% This script takes in Z, Y, V and S, and combine them with info on my
% sectors of interest (Primary aluminium, Alumina and Bauxite). The four
% arrays are expanded to include these new sectors. Following this
% expansion, the arrays are transformed to a per-unit basis by dividing
% them by the total output, X.
%
% Author: Kjartan Steen-Olsen

%% Establishing basic inputs
framework = 'Framework.xlsx';                                               % Info on which regions, sectors and stressors that are used (by all of us)
calcs = 'Calculations_Kjartan.xlsx';                                        % Some basic calculations for my project

[dummy,stressors] = xlsread(framework,'Info','B4:B11');                     % Stressor names
[dummy,regions] = xlsread(framework,'Info','C4:C34');                       % Region names
[dummy,sectors] = xlsread(framework,'Info','F4:F67');                       % Sector names (EU)
[dummy,ROWsectors] = xlsread(framework,'Info','D4:D65');                    % Sector names (ROW)
clear dummy

strs = size(stressors,1);                                                   % Number of stressors (8)
regs = size(regions,1);                                                     % Number of regions (31)
scts = size(sectors,1);                                                     % Number of sectors (64)
sctsW = size(ROWsectors,1);                                                 % Number of ROWsectors (62)

load('Z_ROW.mat')                                                           % Dimensions: [1968 x 1968]
load('Y_ROW.mat')                                                           % Dimensions: [1968 x 93]
load('V_EU.mat')                                                            % Dimensions: [7 x 1472]
load('S_tot.mat')                                                           % Dimensions: [8 x 1984]
load('S_hh_EU.mat')                                                         % Dimensions: [8 x 23]

%% Expanding matrices using dummy (0) sectors and regions:
[Z_start,Y_start] = addummyscts(Z_ROW,y_ROW,regs,scts);
V_start = V_EU;
V_start(:,1473:1984) = 0;
S_start = S_tot;
% New dimensions: 64 sectors x 31 regions = 1984 region-sectors

%% Calculating final consumption
%fc = determinefc(Y_elrowdisag,regions);
fc = sum(Y_start,2);

%% Calculating total purchases and sales for each region-sector:
[outputs,inputs] = determineIO(Z_start,fc,calcs,regs,scts);

%% Reading info about desired new sectors from the calculations sheet, storing relevant parameters:
newsectors_basis = estbl_new_scts(calcs,outputs);

%% Creating one column and one row for each new sector and for every region based on info from the above (shares):
[newsectors_purchases,newsectors_sales,newsectors_fc,newsectors_VA,...
    newsectors_em] = makesectors(newsectors_basis,Z_start,Y_start,...
    V_start,S_start,regs,scts);

%% Updating original Z, fc, V and S, i.e. subtracting extracted data
[Z_upd,Y_upd,V_upd,S_upd] = deductsectors(Z_start,Y_start,V_start,...
    S_start,newsectors_purchases,newsectors_sales,newsectors_fc,...
    newsectors_VA,newsectors_em,newsectors_basis,regs,scts);

%% Expanding Z, fc and S by adding new columns and rows for each region-sector
[Z_disag,Y_disag,V_disag,S_disag] = expandZ(Z_upd,Y_upd,V_upd,...
    S_upd,newsectors_purchases,newsectors_sales,newsectors_fc,...
    newsectors_VA,newsectors_em,regs,scts);

%% Fixing zero and negative values:
Z_disag(Z_disag<1e-6)=0;
Y_disag(Y_disag<1e-6)=1e-6;

%% Updating sector list
newsectors = updatesectors(sectors,newsectors_basis);
newscts = size(newsectors,1);

%% Calculating total sales, x:
outputs_new = determineIO(Z_disag,Y_disag,0,regs,newscts);
for i = 1:regs
    X(size(newsectors,1)*(i-1)+1:size(newsectors,1)*i,1) =...
        outputs_new(i,:)';
end

%% Constructing A, v and s (i.e. normalizing Z and S)
[A_temp,v,s_temp] = ZtoA(Z_disag,Y_disag,V_disag,S_disag);

%% Updating A_ff and s_f
[A,s,Z,Y,V,S] = makeff(A_temp,s_temp,Z_disag,Y_disag,V_disag,...
    S_disag,newsectors_basis,calcs,regs,newscts);

%% Iterative procedure to make A yield correct Al output
% Alu output from x=L*y were 33% too large:
A(65:67:end,:) = A(65:67:end,:)*0.73;

%% Adjusting electricity emissions:
% new_s_el

%% Characterization matrix:
C = [0.023,1,0.296,0,0,0,0,0;0,0,0,0,0.5,0,0,1.2];
%GWP (ktons CO2-eq), AP (tons SO2-eq) for CH4, CO2, N2O, NH3, NOx, CO, NMVOC, SOx
s_temp = s;
s_temp(isnan(s_temp)) = 0;
Cs = C*s_temp;

%% Constructing the Leontief inverse:
I = eye(size(A,1));
L = inv(I-A);
E_pu_fd = diag(Cs(1,:))*L;                                                   % E_ij gives ktons of CO2 from sector i per unit of final demand of sector j's output
E_pu_fd_f = E_pu_fd(:,[65:3*31+64]+(67-3)*floor([0:3*31-1]/3));

%% Clearing variables..
% clear sectors sctsW Z_start Y_start V_start S_start inputs Z_upd Y_upd...
%     V_upd S_upd Z_disag Y_disag V_disag S_disag A_temp s_temp V_EU S_tot...
%     i Z_ROW y_ROW inputs outputs I ans

% Exporting outputs
save 'MainK_results.mat' E_pu_fd_f A S X Y Z s v newsectors_basis
